import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;

/**
 * Dungeon game main program.
 * Reads in the dungeon configuration from standard input.
 * Usage:
 *  java DungeonGame d0.txt
 *  
 * Do NOT modify this file. 
 *  
 * @author Keith Vertanen
 * @author Michele Van Dyne - file input modifications
 *
 */
public class DungeonGame
{
	/**
	 * Main method for the game control and game loop
	 * 
	 * @param args file name for the configuration file
	 */
	public static void main(String [] args)
	{
		Config config = new Config();
		try
		{
			Scanner file = new Scanner(new File(args[0]));

			// Read in the size and number of keys/gems
			int width   = file.nextInt();
			int height  = file.nextInt();
			int numKeys = file.nextInt();
			int numGems = file.nextInt();

			// Prepare the standard draw window, without a border for pixel perfect drawing
			StdDraw.setCanvasSize(config.cellPixels() * width, config.cellPixels() * height);
			StdDraw.setXscale(0.0, config.cellPixels() * width);     
			StdDraw.setYscale(0.0, config.cellPixels() * height);

			// Construct the dungeon layout
			Dungeon dungeon = new Dungeon(config, width, height);

			// Read in the initial location of the hero
			int heroX = file.nextInt();
			int heroY = file.nextInt();
			Hero hero = new Hero(config, heroX, heroY);
			dungeon.addHero(hero);

			// Setup all the cells in the dungeon
			for (int y = 0; y < height; y++)
			{
				for (int x = 0; x < width; x++)
				{
					dungeon.set(x, height - y - 1, file.next()); 
				}
			}
			file.close();

			// Add the right number of keys and gems at random locations
			for (int i = 0; i < numKeys; i++)
				dungeon.addItem(Item.KEY);
			for (int i = 0; i < numGems; i++)
				dungeon.addItem(Item.GEM);  

			StdDraw.show(0);
			dungeon.draw();
			StdDraw.show(0);

			while (true)
			{
				if (StdDraw.hasNextKeyTyped())
				{               
					Action action = config.mapKeyToAction(StdDraw.nextKeyTyped());
					dungeon.execute(action);
					dungeon.draw();
				}      
				StdDraw.show(100);
			}

		}
		catch (FileNotFoundException e)
		{
			System.out.println("File not found.");
		}

	}   
}
